from sys import exit
 
#Panda section
from panda3d.core import *
from direct.task import Task

#DMK section
from DobjBase import DobjBase
from DMKVar import DMKVar
from DMKMaterialManager import DMKMaterialManager
from DMKPhysics import DMKDobjPhysicsManager
from DMKLights import DMKDobjLightManager
from DMKInstanceManager import DMKInstanceManager
from DMKDobjShaderManager import DMKDobjShaderManager

class Dobj(DobjBase):
    """Class Dobj is the base class for DemoMaker displayable Dobj."""
    def __init__(self,displayname,runmode,dbjid):
        DobjBase.__init__(self,displayname,dbjid)

        self.oldtasktime = 0.0
        self.np = NodePath(self.GetDobjId()) #The nodepath of the Dobj
        self.DMKMaterialManager = DMKMaterialManager("DMKMaterialManager",True)
        self.DMKDobjPhysicsManager = DMKDobjPhysicsManager()
        self.PhysicsNode = None
        self.DMKDobjLightManager = DMKDobjLightManager.DMKDobjLightManager()
        self.DMKInstanceManager = DMKInstanceManager() 
        self.DMKDobjManager = None #Will be used to store the DobjManager and retrieve other Dobj for parenting
        self.DMKEmptyNP = None #Will be used to reparent to the Dobj but without inheriting from Scale TO DO : To variabilize ??
        self.DMKShaderManager = DMKDobjShaderManager()
     
        self.vardictionnary["StartTime"]= DMKVar("Time","float",0.0,0.0,True,None,None)
        self.vardictionnary["Duration"]= DMKVar("Time","float",160.0,160.0,True,None, None)
        self.vardictionnary["Position"]= DMKVar("Main","Point3",(10,20,30),(-0.6,0.8,0),True,NodePath.setPos,NodePath.getPos)
        self.vardictionnary["PositionFromAsset"]= DMKVar("Main","bool",False,False,True,None,None) 
        self.vardictionnary["Scale"]= DMKVar("Main","float",1.0,1.0,True,NodePath.setScale,NodePath.getScale)
        self.vardictionnary["ScaleFromAsset"]= DMKVar("Main","bool",True,True,True,None,None) 
        self.vardictionnary["Hpr"]= DMKVar("Main","Point3",(0,0,0),(0,0,0),True,NodePath.setHpr,NodePath.getHpr)
        self.vardictionnary["HprFromAsset"]= DMKVar("Main","bool",True,True,True,None,None) 
        self.vardictionnary["ParentMode"] = DMKVar("Parent","bool",False,False,True,None,None) 
        self.vardictionnary["ParentDobj"] = DMKVar("Parent","FromDbjManager"," "," ",True,None,None) 
        self.vardictionnary["InheritHpr"] = DMKVar("Parent","bool",True,True,True,None,None) 
        self.vardictionnary["InheritScale"] = DMKVar("Parent","bool",True,True,True,None,None) 
        #Physics is managed in the DobjPhysicsManager
        #Main TreeNodes of displayable Dobj. Used to store the TreenodeitemId. To DO : Treenode "Children"
        self.treenodedic["Instances"] = None
        self.treenodedic["Material"] = None
        self.treenodedic["Lights"] = None
        self.treenodedic["Shader"] = None
        self.treenodedic["Physics"] = None
        self.treenodedic["Audio"] = None

    def Initialize(self):
        if self.GetValueByProperty("isActive")==False:
            return False
        else:
            if self.GetValueByProperty("ParentMode") == True:
                if self.DMKDobjManager is not None:
                    fpsnp = self.DMKDobjManager.GetNPfromDobjDisplayName(self.GetValueByProperty("ParentDobj"))
                    if (fpsnp!=NodePath.notFound()) and (fpsnp is not None): 
                        self.__AttachToParent(fpsnp)
                    else:
                        taskMgr.add(self.__CheckToParentDobjTask,"CheckToParentDobjTask-"+self.GetDobjId())

    def GetDobjStartTime(self):
        return self.vardictionnary["StartTime"].value

    def GetDobjDuration(delf):
        return self.vardictionnary["Duration"].value

    def CheckDobjLifeTime(self,task):
        if task.time  >= self.oldtasktime + self.vardictionnary["Duration"].value:
            self.Stop()
            return Task.done
        return Task.cont

    def SetValueByProperty(self,propname, value):
        if self.vardictionnary.has_key(propname) == True:
            if (self.np is not None) and (value is not None):
                self.vardictionnary[propname].SetValue(self.np,value)
                return True
        else:
            return False
     
    def StartDobjRendering(self,task):
        self.oldtasktime = task.time
        self.Initialize()

#AssetManager part
    def GetDemoAssetManager(self,assetmanager):
        """Retrieving the DemoLevel AssetManager."""
        self.DemoAssetManager = assetmanager

#Physics part
    def GetWorldPhysics(self,world):
        """Retrieving the DemoLevel Physics World."""
        self.world = world

    def GetDobjPhysicsManager(self):
        """Return the Physics Manager at the Dobj level."""
        return self.DMKDobjPhysicsManager

    def InitializePhysicsManager(self,model,asset):
        """Initialize the DobjPhysicsManager. Return the Physics Node. Or self.PhysicsNode will stay 'None'."""
        if self.DMKDobjPhysicsManager is not None:
            if self.DMKDobjPhysicsManager.GetValueByProperty("isActive")==True: 
                return self.DMKDobjPhysicsManager.Initialize(self.world,self.dbjid,model,asset)

#Instancing part
    def InitializeInstanceManager(self,pandafile):
        """Returns True if Instances are ok, False otherwise."""
        return self.DMKInstanceManager.Initialize(pandafile,self)

#Material part
    def GetMaterialManager(self):
        return self.DMKMaterialManager

#Light part
    def GetDobjLightManager(self):
        return self.DMKDobjLightManager

    def InitializeDobjLightManager(self):
        if hasattr(self,"DMKDobjLightManager") == True:
            if self.DMKDobjLightManager.HasLight() == False:
                self.np.setLightOff()
            if self.DMKDobjLightManager.HasFog() == False:
                self.np.clearFog()	
#Shader Part
    def InitializeShaderManager(self):
        #self.DMKShaderManager = DMKDobjShaderManager()
        self.DMKShaderManager.Initialize(self.np)

#DobjManager Part (for parenting)
    def GetDobjManager(self,dobjmanager):
        """Retrieve the DobjManager to use in that dobj"""
        self.DMKDobjManager = dobjmanager

    def __CheckToParentDobjTask(self,task):
        fpsnp = self.DMKDobjManager.GetNPfromDobjDisplayName(self.GetValueByProperty("ParentDobj"))
        if (fpsnp!=NodePath.notFound()) and (fpsnp is not None) and (fpsnp.isEmpty()==False):
           self.__AttachToParent(fpsnp)
           return task.done
        else:
           return task.cont

    def __AttachToParent(self,fpsnp):
        """Internal method to manage the attachment to the Dobj"""
        self.DMKEmptyNP = NodePath("Empty-"+self.GetDobjId())  
        self.DMKEmptyNP.reparentTo(fpsnp)
        if self.GetValueByProperty("InheritScale") == False:
            self.DMKEmptyNP.setScale(self.GetValueByProperty("Scale"))
        if self.GetValueByProperty("InheritHpr") == False:
            self.DMKEmptyNP.setHpr(self.GetValueByProperty("Hpr"))
        self.np.reparentTo(self.DMKEmptyNP)
        #print self.GetDobjDisplayName()+" reparented at "+ str(self.np.getPos()) + "of " + fpsnp.getName()

    def Stop(self):
        taskMgr.remove("CheckToParentDobjTask-"+self.GetDobjId())
        self.DMKMaterialManager.Stop()
        self.DMKControllerManager.Stop()
        self.DMKShaderManager.Stop()
        self.np.removeNode()
        if self.DMKEmptyNP is not None:
            self.DMKEmptyNP.removeNode()
        
     